@@ -20,10 +20,11 @@ copy_dll('clDNNPlugin')
 copy_dll('HeteroPlugin')
 copy_dll('inference_engine')
 copy_dll('inference_engine_ir_reader')
-copy_dll('inference_engine_ir_v7_reader')
+#copy_dll('inference_engine_ir_v7_reader')
 copy_dll('inference_engine_legacy')
 copy_dll('inference_engine_transformations')  # runtime
 copy_dll('inference_engine_lp_transformations')  # runtime
+#copy_dll('inference_engine_preproc')  # runtime
 copy_dll('MKLDNNPlugin')  # runtime
 copy_dll('myriadPlugin')  # runtime
 #copy_dll('MultiDevicePlugin')  # runtime, not used
